use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Aaron
Branch
ChangeNotify
Crane
DROLSKY
DROLSKY's
Dave
Default
Etheridge
Event
File
FreeBSD
Inotify
KQueue
Karen
Linux's
Merijn
OpenBSD
PayPal
Rolsky
Rolsky's
Watcher
arc
autarch
brand
distro
drolsky
ether
lib
userspace
